#!/bin/bash
# © Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

# Starts or Restarts Docker daemon
restart_docker() {
	systemctl restart docker.service > /dev/null 2>&1
	systemctl status docker.service > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        logger -t cis "Unable to start docker daemon.\n"
        exit 1
    fi
}

restart_firewall() {
	systemctl restart SuSEfirewall2.service > /dev/null 2>&1
	if [ $? -ne 0 ]; then
        logger -t cis "Unable to start SuSEfirewall2.\n"
        exit 1
    fi
	yast2 firewall masquerade enable > /dev/null 2>&1
}

disable_ip_tables() {
	DOCKER_SYSCONFIG="/etc/sysconfig/docker"
	if [ ! -f $DOCKER_SYSCONFIG ]; then
		logger -t cis "Docker not configured. Exiting...\n"
		exit 1;
	fi

	DOC_OPTS=`grep "^DOCKER_OPTS" $DOCKER_SYSCONFIG`
	if [ $? -ne 0 ]; then
		echo "DOCKER_OPTS=\"--iptables=false\"" >> $DOCKER_SYSCONFIG
		logger -t cis "Docker OPTS iptables not found, so added to docker config file.\n"
	else
		#DOCKER_OPTS found. checking for iptables options
		echo $DOC_OPTS | grep '\-\-iptables=' > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			#iptables option not found so append to the existing options."
			NEW_DOC_OPTS=`echo $DOC_OPTS | sed  's/\(.*\)"$/\1 --iptables=false"/'`
		fi

		echo $DOC_OPTS | grep "\-\-iptables=false" > /dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo $DOC_OPTS | grep "\-\-iptables=true" > /dev/null 2>&1
            if [ $? -eq 0 ]; then
                #iptables option was true. changinf it to false.
                NEW_DOC_OPTS=`echo $DOC_OPTS | sed  's/\(--iptables=\)true/\1false/'`
            fi
            #Updating the options to config file
            sed -i "s/\(^DOCKER_OPTS=\).*$/$NEW_DOC_OPTS/" $DOCKER_SYSCONFIG > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                logger -t cis "Failed to update docker options.\n"
                exit 1;
            fi
		fi
	fi
}

#Firewall configurations
# Enable ports for CIS external services
# 2377, 7946 - Required for Docker Swarm initialization, heart beat and leader election.
# 2181 - Zookeeper: Non Secure channel
# 2282 - Zookeeper: Secure channel { Front face by Envoy }
# 2888 3888 - Required for Zookeeper initialization, heart beat and leader election.
# 9094 - Kafka: Secure channel
# 9400 - Elasticsearch { Front face by Envoy }
yast2 firewall services add tcpport=2377,7946,2181,2282,2888,3888,9094,9400 zone=EXT > /dev/null 2>&1
yast2 firewall services add udpport=7946,4789 zone=EXT > /dev/null 2>&1

# Disabling IP tables in Docker configuration Options
disable_ip_tables

# Check for SuSEfirewall2 process
restart_firewall

# Check for docker process
restart_docker

BRIDGE_GATEWAY=""

# Collect docker configuration details
BRIDGE_GATEWAY=`docker network inspect bridge | grep -e Gateway | cut -d':' -f2 | sed 's/[",[:space:]]//g'`
if [ -z $BRIDGE_GATEWAY ]; then
    sleep 3	
	restart_docker
	BRIDGE_GATEWAY=`docker network inspect bridge | grep -e Gateway | cut -d':' -f2 | sed 's/[",[:space:]]//g'`
	if [ -z $BRIDGE_GATEWAY ]; then
		logger -t cis "Unable to get docker default bridge gateway.\n"
		exit 1;
	fi
fi

# Envoy configuration path
ES_ENVOY_CONFIG_PATH="/etc/opt/novell/cis/es_envoy.json"
ZK_ENVOY_CONFIG_PATH="/etc/opt/novell/cis/zk_envoy.json"

# For Envoy to route incoming requests, configure envoy configuration json file on docker gateway address
sed -i "s/{address}/${BRIDGE_GATEWAY}/g" ${ZK_ENVOY_CONFIG_PATH}

# Certificate Management 
if ! { [ -e /etc/ssl/servercerts/servercert.pem ] && [ -e /etc/ssl/servercerts/serverkey.pem ] && [ -e /etc/opt/novell/certs/SSCert.pem ]; }; then
	logger -t cis "Server certificates not found"
	exit 1;
fi

mkdir -p /etc/opt/novell/cis/certs/rootCAs
cp /etc/ssl/servercerts/servercert.pem /etc/opt/novell/cis/certs
cp /etc/ssl/servercerts/serverkey.pem /etc/opt/novell/cis/certs
/opt/novell/cis/bin/appendcert -dir /etc/opt/novell/certs/ -leaf /etc/ssl/servercerts/servercert.pem -chain /etc/opt/novell/cis/certs/rootCAs/SSCert.pem

password="changeit"

if [ ! -d /etc/opt/novell/cis/kafka/certs ]; then
    logger -t cis "certs folder not found"
    exit 1;
fi

rm -f /etc/opt/novell/cis/kafka/certs/* > /dev/null 2>&1

openssl pkcs12 -export -in /etc/opt/novell/cis/certs/servercert.pem -inkey /etc/opt/novell/cis/certs/serverkey.pem -out /etc/opt/novell/cis/kafka/certs/server.p12 -name localhost -CAfile /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -caname rootca -password pass:$password; > /dev/null 2>&1
if [ $? -ne 0 ]; then
    logger -t cis "Failed to import the server certificate, private key and CA certificate into PKCS12 keystore"
    exit 1;
fi

keytool -importkeystore -deststorepass $password -destkeypass $password -destkeystore /etc/opt/novell/cis/kafka/certs/server.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass $password -alias localhost; > /dev/null 2>&1
if [ $? -ne 0 ]; then
    logger -t cis "Failed to Covert the PKCS12 type keystore to JKS type keystore"
    exit 1;
fi

keytool -keystore /etc/opt/novell/cis/kafka/certs/server.truststore.jks -storepass $password  -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt > /dev/null 2>&1
if [ $? -ne 0 ]; then
    logger -t cis "Failed to import the CA certificate into JKS type keystore"
    exit 1;
fi

# Load docker images
/bin/bash /opt/novell/cis/zk/bin/zk_service.sh load > /dev/null 2>&1
/bin/bash /opt/novell/cis/kafka/bin/kafka_service.sh load > /dev/null 2>&1
/bin/bash /opt/novell/cis/es/bin/es_service.sh load > /dev/null 2>&1
/usr/bin/docker load -i /var/opt/novell/cis/nginx.tar.gz > /dev/null 2>&1
docker rmi $(docker images --filter "reference=nginx" --filter "before=nginx:1.14" -q) > /dev/null 2>&1
docker rmi $(docker images --filter "reference=nginx" --filter "since=nginx:1.14" -q) > /dev/null 2>&1

# Pre-requistions
/sbin/sysctl -w net.ipv4.ip_forward=1
/sbin/sysctl -w vm.max_map_count=262144
/usr/bin/chmod 777 -R /etc/opt/novell/cis/es/data
systemctl enable oes-cis-dependencies-ha.service
systemctl disable docker.service
