#!/bin/bash
# © Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

if [ $# -eq 0 ]; then
    logger -t cis "Invalid Command. No arguments supplied.\n"
    exit 1;
fi

upgrade() {
    # Check if Docker daemon running
    systemctl status docker.service
    if [ $? -ne 0 ]; then
        logger -t cis "Docker is not running. Starting it. \n"
        systemctl start docker.service
    fi    
    
    /bin/bash /opt/novell/cis/zk/bin/zk_service.sh load 
    /bin/bash /opt/novell/cis/kafka/bin/kafka_service.sh load 
    /bin/bash /opt/novell/cis/es/bin/es_service.sh load 
    /usr/bin/docker load -i /var/opt/novell/cis/nginx.tar.gz 
    docker rmi $(docker images --filter "reference=nginx" --filter "before=nginx:1.14" -q) > /dev/null
    docker rmi $(docker images --filter "reference=nginx" --filter "since=nginx:1.14" -q) > /dev/null

    # Envoy configuration path
    ZK_ENVOY_CONFIG_PATH="/etc/opt/novell/cis/zk_envoy.json"
    BRIDGE_GATEWAY=""

    # Collect docker configuration details
    BRIDGE_GATEWAY=`docker network inspect bridge | grep -e Gateway | cut -d':' -f2 | sed 's/[",[:space:]]//g'`
    if [ -z $BRIDGE_GATEWAY ]; then
        sleep 3	
        restart_docker
        BRIDGE_GATEWAY=`docker network inspect bridge | grep -e Gateway | cut -d':' -f2 | sed 's/[",[:space:]]//g'`
        if [ -z $BRIDGE_GATEWAY ]; then
            logger -t cis "Unable to get docker default bridge gateway.\n"
            exit 1;
        fi
    fi
    # For Envoy to route incoming requests, configure envoy configuration json file on docker gateway address
    sed -i "s/{address}/${BRIDGE_GATEWAY}/g" ${ZK_ENVOY_CONFIG_PATH}

    if [ -d /var/opt/novell/oldcis ]; then
        /bin/bash /var/opt/novell/cis/elastic_move_data.sh createOldES 
        yast2 firewall services add tcpport=2377,7946,2181,2282,2888,3888,9094,9400,9500 zone=EXT > /dev/null 2>&1
        disable_ip_tables
        /sbin/sysctl -w net.ipv4.ip_forward=1
        /sbin/sysctl -w vm.max_map_count=262144
        /usr/bin/chmod 777 -R /etc/opt/novell/cis/oldes/data        
        /usr/bin/chmod 777 -R /etc/opt/novell/cis/es/data
        restart_firewall 
    fi 
    
    echo "Upgrade completed sucessfully."
}

start() {

    # Check if Docker daemon running
    systemctl status docker.service
    if [ $? -ne 0 ]; then
        logger -t cis "Docker is not running.\n"
        exit 1;
    fi

    # Check if Docker Swarm mode enabled
    docker info | grep "Swarm: active"
    if [ $? -ne 0 ]; then
        logger -t cis "Docker Swarm is not active.\n"
        exit 1;
    fi

    # Request to start the HA services for Zookeeper, Kafka & ElasticSearch
    # Get Docker Swarm node count
    NODECOUNT=`docker info --format '{{json .Swarm.Nodes}}'`

    # Check if the services are running already
    docker stack ps cis
    if [ $? -eq 0 ]; then
        logger -t cis "CIS External dependency High Availability stack already running.\n"
        exit 0;
    fi

    # If node count less than 3, exit
    if [ $NODECOUNT -lt 3 ]; then
        logger -t cis "Insufficient Node Count: $NODECOUNT\n"
        exit 1;
    fi

    TEMPLATE=7
    # Find the suitable template file.
    if [ $NODECOUNT -lt 7 ]; then
        TEMPLATE=5
    fi
    if [ $NODECOUNT -lt 5 ]; then
        TEMPLATE=3
    fi

    TEMPLATE_FILE="/etc/opt/novell/cis/cis$TEMPLATE.yml"

    #check if there is node.lock in any of the nodes
    prepareformigrate
        
    # Load the template file
    docker stack deploy -c ${TEMPLATE_FILE} cis
    if [ $? -ne 0 ]; then
        logger -t cis "Failed to deploy the CIS External dependencies High Availability stack.\n"
        exit 1;
    fi

    # Check Run status
    docker stack ps cis
    if [ $? -ne 0 ]; then
        logger -t cis "CIS External dependency High Availability stack failed.\n"
        exit 1;
    fi

    logger -t cis "CIS External dependency High Availability stack Up and Running.\n"    
}

migrate() {
    # if we have to move data
    if [ -d /etc/opt/novell/cis/oldes/ ]; then
        # check if there is a node.lock from previous entry
        if [ ! -d /etc/opt/novell/cis/es/data/nodes/0/indices ]; then
            if [ -f /etc/opt/novell/cis/es/data/nodes/0/node.lock ]; then
                rm -rf /etc/opt/novell/cis/es/data/*
            fi
        fi
        /usr/bin/sh /var/opt/novell/cis/elastic_move_data.sh migrate        

        if [ $? == 0 ]; then
            complete_migrate
        fi
    fi    
}

stop() {
    # Request to stop the HA services for Zookeeper, Kafka & ElasticSearch
    # Check if the services are running already
    docker stack ps cis
    if [ $? -ne 0 ]; then
            logger -t cis "CIS External dependency High Availability stack currently down.\n"
            exit 1;
    fi
    # Initiate Service stop
    docker stack rm cis
    if [ $? -eq 0 ]; then
        logger -t cis "Stopping CIS External dependency High Availability stack.\n"
        exit 0;
    fi
}

prepareformigrate() {
    #check if there is node.lock in any of the nodes
    if [ -d /etc/opt/novell/cis/oldes/ ]; then
        # check if there is a node.lock from previous entry
        if [ ! -d /etc/opt/novell/cis/es/data/nodes/0/indices ]; then
            if [ -f /etc/opt/novell/cis/es/data/nodes/0/node.lock ]; then
                rm -rf /etc/opt/novell/cis/es/data/*
            fi
        fi
    fi
}

complete_migrate() {
    sh /var/opt/novell/cis/elastic_move_data.sh cleanupRemaining /tmp/stackelasticcleanup
    TEMPLATE=7
    NODECOUNT=`docker info --format '{{json .Swarm.Nodes}}'`
    # Find the suitable template file.
    if [ $NODECOUNT -lt 7 ]; then
        TEMPLATE=5
    fi
    if [ $NODECOUNT -lt 5 ]; then
        TEMPLATE=3
    fi      

    for i in $(seq 1 $TEMPLATE); do 
        docker service rm cis_oldelastic$i 2>/dev/null
    done
    docker service rm cis_oldesnginx 2>/dev/null

    #replace yml file
    cp /etc/opt/novell/cis/newcis$TEMPLATE.yml /etc/opt/novell/cis/cis$TEMPLATE.yml   
    rm /etc/opt/novell/cis/nginx3.conf
    rm /etc/opt/novell/cis/nginx5.conf
    rm /etc/opt/novell/cis/nginx7.conf

    logger -t cis "CIS External dependency High Availability stack has Upgraded sucessfully.\n"
    echo "CIS External dependency High Availability stack has Upgraded sucessfully.\n"
}

clean() {
    rm -rf /etc/opt/novell/cis/es/data/* /etc/opt/novell/cis/kafka/kafka-logs/* /etc/opt/novell/cis/zk/data/* /etc/opt/novell/cis/zk/datalog/*
}

disable_ip_tables() {
	DOCKER_SYSCONFIG="/etc/sysconfig/docker"
	if [ ! -f $DOCKER_SYSCONFIG ]; then
		logger -t cis "Docker not configured. Exiting...\n"
		exit 1;
	fi

	DOC_OPTS=`grep "^DOCKER_OPTS" $DOCKER_SYSCONFIG`
	if [ $? -ne 0 ]; then
		echo "DOCKER_OPTS=\"--iptables=false\"" >> $DOCKER_SYSCONFIG
		logger -t cis "Docker OPTS iptables not found, so added to docker config file.\n"
	else
		#DOCKER_OPTS found. checking for iptables options
		echo $DOC_OPTS | grep '\-\-iptables=' > /dev/null
		if [ $? -ne 0 ]; then
			#iptables option not found so append to the existing options."
			NEW_DOC_OPTS=`echo $DOC_OPTS | sed  's/\(.*\)"$/\1 --iptables=false"/'`
		fi

		echo $DOC_OPTS | grep "\-\-iptables=false" > /dev/null
		if [ $? -ne 0 ]; then
			echo $DOC_OPTS | grep "\-\-iptables=true" > /dev/null
            if [ $? -eq 0 ]; then
                #iptables option was true. changinf it to false.
                NEW_DOC_OPTS=`echo $DOC_OPTS | sed  's/\(--iptables=\)true/\1false/'`
            fi
            #Updating the options to config file
            sed -i "s/\(^DOCKER_OPTS=\).*$/$NEW_DOC_OPTS/" $DOCKER_SYSCONFIG
            if [ $? -ne 0 ]; then
                logger -t cis "Failed to update docker options.\n"
                exit 1;
            fi
		fi
	fi
}

restart_docker() {
	systemctl restart docker.service
	systemctl status docker.service
    if [ $? -ne 0 ]; then
        logger -t cis "Unable to start docker daemon.\n"
        exit 1
    fi
}

restart_firewall() {
	systemctl restart SuSEfirewall2.service
	if [ $? -ne 0 ]; then
        logger -t cis "Unable to start SuSEfirewall2.\n"
        exit 1
    fi
	yast2 firewall masquerade enable
}

prepare() {
    systemctl status oes-cis-fluentbit.service
    if [ $? -eq 0 ]; then
        systemctl stop oes-cis-fluentbit.service
    fi
    systemctl disable oes-cis-fluentbit.service
    yast2 firewall services add tcpport=2377,7946,2181,2282,2888,3888,9094,9400,9500 zone=EXT > /dev/null 2>&1
    yast2 firewall services add udpport=7946,4789 zone=EXT > /dev/null 2>&1
    disable_ip_tables
    /sbin/sysctl -w net.ipv4.ip_forward=1
    /sbin/sysctl -w vm.max_map_count=262144
    if [ -d /etc/opt/novell/cis/oldes ]; then
        /usr/bin/chmod 777 -R /etc/opt/novell/cis/oldes/data
    fi
    /usr/bin/chmod 777 -R /etc/opt/novell/cis/es/data
    restart_firewall
    sleep 60
    restart_docker
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    restart)
        stop
        sleep 5
        start
        ;;
    prepare)
        prepare
        ;;
    upgrade)
        upgrade
        ;;
    prepareformigrate)
        prepareformigrate
        ;;
    migrate)
        migrate
        ;;
    completemigrate)
        complete_migrate
        ;;
    status)
        docker stack ps cis
        if [ $? -eq 0 ]; then
            logger -t cis "CIS External dependency High Availability stack up and running.\n"
            exit 0;
        else
            logger -t cis "CIS External dependency High Availability stack currently down.\n"
            exit 1;
        fi
        ;;
    clean)
        clean
        ;;
esac
